{$A+,B-,D-,E-,F+,G-,I-,L-,N-,O-,P-,Q-,R-,S-,T-,V-,X+}
(*===================================================================*)
(*                          BGIOVR.PAS                               *)
(*                     BGI-Treiber im Overlay                        *)
(*              Diese Unit darf nicht im Overlay sein!               *)
(*===================================================================*)

UNIT BGIOvr;

INTERFACE

USES
  Graph, Overlay, Trip_, Litt_, Sans_, Goth_, Simp_, TSCR_, LCom_,
  Euro_, Scri_, Bold_, BGIDrv;

CONST
  ScriptFont        =  5;        (* die erweiterten Fonts von Turbo/ *)
  SimplexFont       =  6;        (* Borland Pascal 7.0               *)
  TriplexScriptFont =  7;
  ComplexFont       =  8;
  EuropeanFont      =  9;
  BoldFont          = 10;

FUNCTION  LoadDriver(GraphDriver: INTEGER): INTEGER;
PROCEDURE OvrMinimalBuf;
PROCEDURE SetTextStyle(Font, Direction, CharSize: WORD);

IMPLEMENTATION

(*-------------------------------------------------------------------*)
(*                        Laden eines Treibers                       *)
(*-------------------------------------------------------------------*)
                                          (* Speicher fr einen BGI- *)
VAR                                       (* Treiber reservieren:    *)
  DriverBuffer: ARRAY[1..6500] OF BYTE;   (* grter Treiber         *)

FUNCTION LoadDriver(GraphDriver: INTEGER): INTEGER;
(* Auswahl und Laden des BGI-Treibers mit der Nummer GraphDriver   *)
BEGIN
  LoadDriver := RegisterDriver(GraphDriver, SizeOf(DriverBuffer),
                               @DriverBuffer);
END;

PROCEDURE OvrMinimalBuf;
(* Setzt den Overlay-Buffer auf die kleinstnotwendige Gre, auch    *)
(* wenn diese kleiner ist, als die des Treiber-Unit-Moduls. Das wird *)
(* ja nur einmal ganz am Anfang geladen! Der Heap mu leer sein!     *)
VAR
  OvrMem, OvrMin: LONGINT;
  OvrList       : WORD;
BEGIN
  IF OvrResult <> 0 THEN Exit; (* Segment des Steuerblocks des       *)
                               (* Moduls aus der Treiber-Unit.       *)
  OvrList := OvrCodeList;      (* Anfang der Kette der Steuerblcke  *)
  OvrMin := 0;                 (* relativ zum Programmanfang.        *)
  WHILE OvrList <> 0 DO        (* Der Kette entlang hangeln ...      *)
  BEGIN
    OvrList := PrefixSeg + $10 + OvrList;  (* Segm. des Steuerblocks *)
                                           (* wenn Programm geladen. *)
    IF OvrList <> Seg(RegisterDriver) THEN (* Driver-Unit-Modul wird *)
    BEGIN                                  (* nicht untersucht.      *)
      OvrMem := WORD(Ptr(OvrList, $08)^)   (* Gre des Moduls + 32  *)
              + WORD(Ptr(OvrList, $0A)^) + $20;
      IF OvrMin < OvrMem THEN OvrMin := OvrMem;
    END;
    OvrList := WORD(Ptr(OvrList, $0E)^);  (* ... am Kettenende = 0   *)
  END;
  IF OvrGetBuf > OvrMin THEN      (* Verkleinern des Overlay-Buffers *)
  BEGIN                           (* und Neusetzen des Heap-Anfangs  *)
    OvrClearBuf;
    OvrHeapSize := OvrMin DIV $10;             (* ... in Paragraphen *)
    OvrSetBuf(OvrHeapSize * $10);
  END;
END;

(*-------------------------------------------------------------------*)
(*                        Laden eines Fonts                          *)
(*-------------------------------------------------------------------*)
TYPE
  FontArray = ARRAY[1..$5000] OF BYTE;               (* grter Font *)
                            (* --> Speicher fr einen BGI-Vektorfont *)
CONST
  FontArPtr : ^FontArray = NIL;
  FontLength: WORD       = 0;

FUNCTION LoadFont(Font: INTEGER): INTEGER;
VAR
  FontPtr: Pointer;
BEGIN
  IF Font IN [1..10] THEN
  BEGIN
    CASE Font OF
      TriplexFont:       FontPtr := ReadTriplexFont;
      SmallFont:         FontPtr := ReadSmallFont;
      SansSerifFont:     FontPtr := ReadSansSerifFont;
      GothicFont:        FontPtr := ReadGothicFont;
      SimplexFont:       FontPtr := ReadSimplexFont;
      BoldFont:          FontPtr := ReadBoldFont;
      EuropeanFont:      FontPtr := ReadEuropeanFont;
      TriplexScriptFont: FontPtr := ReadTriplexScriptFont;
      ScriptFont:        FontPtr := ReadScriptFont;
      ComplexFont:       FontPtr := ReadComplexFont;
    END;
    IF Seg(FontPtr^) <> OvrLoadList THEN Exit;            (* Fehler! *)
    IF FontArPtr <> NIL THEN FreeMem(FontArPtr, FontLength);
    FontPtr := Ptr(WORD(Ptr(Seg(FontPtr^), $10)^),
                   WORD(Ptr(Seg(FontPtr^), Succ(Ofs(FontPtr^)))^));
    FontLength := WORD(Ptr(Seg(FontPtr^), $08)^);
    GetMem(FontArPtr, FontLength);
    Move(FontPtr^, FontArPtr^, FontLength);
    LoadFont := RegisterBGIFont(FontArPtr);
  END;
END;

PROCEDURE SetTextStyle(Font, Direction, CharSize: WORD);
BEGIN
  IF Font IN [TriplexFont..BoldFont] THEN
  BEGIN
    IF LoadFont(Font) >= 0 THEN
      Graph.SetTextStyle(Font, Direction, CharSize);
  END
  ELSE
    Graph.SetTextStyle(Font, Direction, CharSize);
END;

END.

(*===================================================================*)
