UNIT CDAudio;

INTERFACE

USES Objects;

VAR
  DeviceID: WORD;

PROCEDURE ShowError (ErrorCode: LONGINT);
FUNCTION cdOpen: LONGINT;
FUNCTION cdMediaCheck: BOOLEAN;
FUNCTION cdStop: LONGINT;
FUNCTION cdPause: LONGINT;
FUNCTION cdClose: LONGINT;
FUNCTION cdGoto (Track, Min, Sec: BYTE): LONGINT;
FUNCTION cdSeek (Track, Min, Sec: BYTE): LONGINT;
FUNCTION cdPlay (StartTrack, EndTrack: BYTE): LONGINT;
PROCEDURE GetTMSF (VAR Track, Min, Sec, Frame: BYTE);
FUNCTION HowManyTracks: LONGINT;
FUNCTION GetTrackLen (TrackNo: LONGINT): LONGINT;

IMPLEMENTATION

USES WinTypes, WinProcs, MMSystem, Strings;

VAR
  Paused, AlreadyClosed: BOOLEAN;

PROCEDURE ShowError (ErrorCode: LONGINT);
CONST
  MessageLength = 128;
VAR
  Buffer: ARRAY [0..MessageLength] OF CHAR;
BEGIN
  IF mciGetErrorString (ErrorCode, pChar(@Buffer), MessageLength) THEN
    MessageBox (0, Buffer, 'Fehler aufgetreten', mb_OK);
END;

PROCEDURE MSF;
{ get information in minute / second / frame format }
VAR
  mciSetParms: TMCI_Set_Parms;
BEGIN
  mciSetParms.dwTimeFormat := mci_Format_MSF;
  mciSendCommand (DeviceID, mci_Set, mci_Set_Time_Format, LONGINT(@mciSetParms))
END;

PROCEDURE TMSF;
{ get information in track / minute / second / frame format }
VAR
  mciSetParms: TMCI_Set_Parms;
  ReturnCode: LONGINT;
BEGIN
  mciSetParms.dwTimeFormat := mci_Format_TMSF;
  ReturnCode := mciSendCommand (DeviceID, mci_Set, mci_Set_Time_Format, LONGINT(@mciSetParms));
  IF (ReturnCode <> 0) THEN ShowError (ReturnCode);
END;

FUNCTION GetNow: LONGINT;
VAR
  mciStatus: TMCI_Status_Parms;
  ReturnCode: LONGINT;
BEGIN
  mciStatus.dwItem := mci_Status_Position;
  mciStatus.dwReturn := 0;
  ReturnCode := mciSendCommand (DeviceID, mci_Status, mci_Status_Item, LONGINT(@mciStatus));
  IF (ReturnCode <> 0) THEN ShowError (ReturnCode);
  GetNow := mciStatus.dwReturn;
END;

PROCEDURE GetTMSF (VAR Track, Min, Sec, Frame: BYTE);
VAR
  WhereAreWe: LONGINT;
BEGIN
  IF (DeviceID <> 0) THEN BEGIN
    WhereAreWe := GetNow;
    Track := Lo (LoWord(WhereAreWe));
    Min := Hi (LoWord(WhereAreWe));
    Sec := Lo (HiWord(WhereAreWe));
    Frame := Hi (HiWord(WhereAreWe));
  END ELSE BEGIN
    Track := 0;
    Min := 0;
    Sec := 0;
    Frame := 0;
  END;
END;

FUNCTION HowManyTracks: LONGINT;
VAR
  mciStatus: TMCI_Status_Parms;
  ReturnCode: LONGINT;
BEGIN
  WITH mciStatus DO
  BEGIN
    dwCallback := 0;
    dwReturn := 0;
    dwItem := mci_Status_Number_Of_Tracks;
    dwTrack := 0;
  END;
  ReturnCode := mciSendCommand (DeviceID, mci_Status, mci_Status_Item, LONGINT(@mciStatus));
  IF (ReturnCode <> 0) THEN ShowError (ReturnCode);
  HowManyTracks := mciStatus.dwReturn;
END;

FUNCTION cdMediaCheck: BOOLEAN;
VAR
  mciStatus: TMCI_Status_Parms;
  ReturnCode: LONGINT;
BEGIN
  cdMediaCheck := FALSE;
  WITH mciStatus DO
  BEGIN
    dwCallback := 0;
    dwReturn := 0;
    dwItem := mci_Status_Media_Present;
    dwTrack := 0;
  END;
  ReturnCode := mciSendCommand (DeviceID, mci_Status, mci_Status_Item, LONGINT(@mciStatus));
  IF (ReturnCode = 0) THEN cdMediaCheck := TRUE;
END;

FUNCTION cdOpen: LONGINT;
VAR
  mciOpen: TMCI_Open_Parms;
BEGIN
  FillChar (mciOpen, SizeOf(TMCI_Open_Parms), 0);
  mciOpen.lpstrDeviceType := pChar (mci_DevType_CD_Audio);
  cdOpen := mciSendCommand (0, mci_Open,
            mci_Open_Type OR mci_Open_Type_ID OR mci_Open_Shareable,
            LONGINT(@mciOpen));
  DeviceID := mciOpen.wDeviceID;
  TMSF;
  Paused := FALSE;
END;

FUNCTION cdStop: LONGINT;
BEGIN
  cdStop := mciSendCommand (DeviceID, mci_Stop, mci_Wait, 0);
END;

FUNCTION cdPause: LONGINT;
BEGIN
  IF NOT Paused
    THEN BEGIN
      cdPause := mciSendCommand (DeviceID, mci_Pause, mci_Wait, 0);
      Paused := TRUE;
    END
    ELSE BEGIN
      cdPause := mciSendCommand (DeviceID, mci_Play, mci_Wait, 0);
      Paused := FALSE;
    END;
END;

FUNCTION cdClose: LONGINT;
BEGIN
  cdClose := mciSendCommand (DeviceID, mci_Close, 0, 0);
END;

FUNCTION cdSeek (Track, Min, Sec: BYTE): LONGINT;
VAR
  mciSeek: TMCI_Seek_Parms;
BEGIN
  mciSeek.dwCallback := 0;
  mciSeek.dwTo := MCI_Make_TMSF (Track, Min, Sec, 0);
  cdSeek := mciSendCommand (DeviceID, mci_Seek, mci_To, LONGINT(@mciSeek));
END;

FUNCTION cdGoto (Track, Min, Sec: BYTE): LONGINT;
VAR
  mciPlay: TMCI_Play_Parms;
  LastTrack: LONGINT;
BEGIN
  mciPlay.dwFrom := MCI_Make_TMSF (Track, Min, Sec, 0);
  LastTrack := HowManyTracks;
  mciPlay.dwTo := MCI_Make_TMSF (LastTrack, mci_MSF_Minute(GetTrackLen(LastTrack)),
                                            mci_MSF_Second(GetTrackLen(LastTrack)), 0);
  cdGoto := mciSendCommand (DeviceID, mci_Play, mci_From, LONGINT(@mciPlay));
END;

FUNCTION cdPlay (StartTrack, EndTrack: BYTE): LONGINT;
BEGIN
  cdPlay := cdGoto (StartTrack, 0, 0);
END;

FUNCTION GetTrackLen (TrackNo: LONGINT): LONGINT;
VAR
  ReturnCode: LONGINT;
  mciStatus: TMCI_Status_Parms;
BEGIN
  mciStatus.dwTrack := TrackNo;
  mciStatus.dwCallback := 0;
  mciStatus.dwReturn := 0;
  mciStatus.dwItem := mci_Status_Length;
  GetTrackLen := mciSendCommand (DeviceID, mci_Status,
                 mci_Status_Item OR mci_Track, LONGINT(@mciStatus));
  GetTrackLen := mciStatus.dwReturn
END;

BEGIN
  DeviceID := 0;
END.
