{*****************************************************}
{                                                     }
{ Example program of Owner Draw Listboxes containing  }
{ Checkboxes. Both single column and a multiple       }
{ column listboxes are here.                          }
{                                                     }
{ Developed by author and released into the public    }
{ domain. Code is Alpha version. No warranty and used }
{ at own descretion.                                  }
{                                                     }
{ Thought others might be interested in hybrid control}
{ examples since there is much floating around in     }
{ in pascal.                                          }
{                                                     }
{ Thanks to Mark Gamber, whose C Code, and Steve      }
{ Leonard, whose Pascal Code, were the building blocks}
{ of this example.                                    }
{                                                     }
{*****************************************************}



program ListTest;

uses WinTypes, WinProcs, Objects, OWindows, ODialogs, BWCC, Strings, TListChk;

{$R LISTTEST.RES}

const id_SingleColumn = 101;
      id_Switch       = 102;
      id_MultiColumn  = 103;

type

    PApp = ^TApp;
    TApp = object(TApplication)
     procedure InitMainWindow; virtual;
    end;

    PMainWindow = ^TMainWindow;
    TMainWindow = object(TDlgWindow)
     SL: PListChkBox;
     ML: PListChkBox;
     constructor Init(AParent: PWindowsObject; AName: PChar);
     procedure   SetupWindow; virtual;
     procedure   CreateItems(NumItems: integer);
     procedure   DestroyItems;
     procedure   WMCommand(var Msg: TMessage); virtual wm_First+wm_Command;
     procedure   WMMeasureItem(var Msg: TMessage); virtual wm_First+wm_MeasureItem;
     procedure   WMDrawItem(var Msg: TMessage); virtual wm_First+wm_DrawItem;
     procedure   DrawItem(DrawItemStruct: PDrawItemStruct); virtual;
     procedure   IDSwitch(var Msg: TMessage); virtual id_First+id_Switch;
    private
     NumListItems: integer;
    end;


constructor TMainWindow.Init(AParent: PWindowsObject; AName: PChar);
begin
     inherited Init(AParent, AName);
     SL := New(PListChkBox, InitResource(@Self, id_SingleColumn));
     ML := New(PListChkBox, InitResource(@Self, id_MultiColumn));
end;

procedure TMainWindow.SetupWindow;
begin
     inherited SetupWindow;
     {LB_SETCOLUMNWIDTH sent here for convinence, would have to compute}
     {string width in pixels using GetTextExt, etc, to be safe}
     SendMessage(ML^.HWindow, LB_SETCOLUMNWIDTH, 140, 0);
     CreateItems(12);
end;

procedure TMainWindow.CreateItems(NumItems: integer);
var i: integer;
    ListRect: TRect;
    str: array[0..25] of char;
begin
     NumListItems := NumItems;
     for i := 0 to NumItems do
     begin
          wvsprintf(str, 'Check Box 1 #%d', i);
          SL^.AddString(Str);
          SendMessage(SL^.HWindow, LB_GETITEMRECT, i, Longint(@ListRect));
          CreateWindow('BorCheck', str, WS_CHILD or WS_VISIBLE or BS_AUTOCHECKBOX,
            ListRect.left, ListRect.Top, 14 , 16, SL^.HWindow, i + 1000, HInstance, nil);
          wvsprintf(str, 'Check Box 2 #%d', i);
          ML^.AddString(Str);
          SendMessage(ML^.HWindow, LB_GETITEMRECT, i, Longint(@ListRect));
          CreateWindow('BorCheck', str, WS_CHILD or WS_VISIBLE or BS_AUTOCHECKBOX,
            ListRect.left, ListRect.Top, 14 , 16, ML^.HWindow, i + 2000, HInstance, nil);
     end;
end;

procedure TMainWindow.DestroyItems;
var i: integer;
begin
     SL^.ClearList;
     ML^.ClearList;
     for i := 0 to NumListItems do
     begin
      DestroyWindow(GetDlgItem(SL^.HWindow, i + 1000));
      DestroyWindow(GetDlgItem(ML^.HWindow, i + 2000));
     end;
end;

procedure TMainWindow.WMCommand(var Msg: TMessage);
begin
     if (Msg.lParamHi = BN_Clicked) and (Msg.wParam >999) then
      if GetParent(Msg.lParamLo) = SL^.HWindow then
       SL^.SetSelIndex(Msg.wParam-1000)
      else
       ML^.SetSelIndex(Msg.wParam-2000);
     inherited WMCommand(Msg);
end;

procedure TMainWindow.WMMeasureItem(var Msg: TMessage);
var DC: HDC;
    TM: TTextMetric;
begin
     DC := GetDC(HWindow);
     if GetTextMetrics(DC, TM) then
      with TM, PMeasureItemStruct(Msg.lParam)^ do
       ItemHeight := tmHeight + tmExternalLeading;
     ReleaseDC(HWindow, DC);
end;

procedure TMainWindow.WMDrawItem(var Msg: TMessage);
var Selected: Boolean;
    OldColor: TColorRef;
    OldBKGnd: TColorRef;
    Brush: HBrush;
begin
    with PDrawItemStruct(Msg.lParam)^ do
     begin
       if (ItemAction and (oda_DrawEntire or oda_Select)) <> 0 then
       begin
         Selected := (ItemState and ods_Selected) <> 0;
         if Selected then
         begin
           OldColor := SetTextColor(hDC, rgb(0,0,0));
           OldBKGnd := SetBKColor(hDC, rgb(0,0,0));
           Brush    := CreateSolidBrush(rgb(0,128,128));
         end
         else
           Brush := CreateSolidBrush(rgb(192,192,192));
         rcItem.left := rcItem.left+14;
         FillRect(hDC, rcItem, Brush);
         DeleteObject(Brush);
         Drawitem(PDrawItemStruct(Msg.lParam));

         if Selected then
         begin
           SetTextColor(hDC, OldColor);
           SetBKColor(hDC, OldBKGnd);
         end;
       end;
     end;
     DefWndProc(Msg);
end;

procedure TMainWindow.DrawItem(DrawItemStruct: PDrawItemStruct);
var name: array[0..25] of char;
begin
     with DrawItemStruct^ do
      with rcItem do
      begin
        SetBkMode( hDC, Transparent ) ;
        SendMessage(hwndItem, LB_GETTEXT, ItemId, longint(@Name));
        TextOut( hDC, Left+10, Top, name, strlen(name)) ;
      end;
end;

procedure TMainWindow.IDSwitch(var Msg: TMessage);
var tmp: integer;
begin
     if NumListItems = 12 then Tmp := 24 else Tmp := 12;
     DestroyItems;
     CreateItems(Tmp);
end;

procedure TApp.InitMainWindow;
begin
     MainWindow := New(PMainWindow, Init(nil, 'LISTTEST'));
end;

var App: TApp;

begin
     App.Init('Test');
     App.Run;
     App.Done;
end.